/* (This is the new BSD license.)
* Copyright (c) 2013-2016, Chris Culy
* All rights reserved.
*
* Redistribution and use in source and binary forms, with or without
* modification, are permitted provided that the following conditions are met:
*     * Redistributions of source code must retain the above copyright
*       notice, this list of conditions and the following disclaimer.
*     * Redistributions in binary form must reproduce the above copyright
*       notice, this list of conditions and the following disclaimer in the
*       documentation and/or other materials provided with the distribution.
*     * Neither the name of the Chris Culy nor the 
*		names of its contributors may be used to endorse or promote 
*		products from this software without specific prior written permission.
*
* THIS SOFTWARE IS PROVIDED BY Chris Culy
* ``AS IS'' AND ANY OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, 
* THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE 
* ARE DISCLAIMED. IN NO EVENT SHALL Chris Culy
* BE LIABLE FOR ANY, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR 
* CONSEQUENTIAL DAMAGES INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE 
* GOODS OR SERVICES; OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER 
* CAUSED AND ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR 
* TORT INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF 
* THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
*/

var srcName; //the name of the file we have loaded, or empty if using the example
var kwic;
var ourModel;


var sortReversed = []; //keep track of whether columns are sorted in ascending or descending order
var rowStrOnRight = false;

var uniqItems;

var itemContextLen = 75; //how much of the full context we'll show when clicking on a hit
var itemHistory = new SearchHistory();

//set up UI stuff
document.addEventListener("keypress",handlePress,false);
var haveWordsElt = true;

  

function setup(loadedText) {
  kwic = new kwicIS.KWIC("realKWIC").rtToLeft(isRightToLeft);
  if (! fromTextInfo) {
    kwic.fieldNames(fldNames).fieldDelim(fldDelim);
  }
  
  kwic.contextLen(kwicContext).rowString(rowStr, rowStrOnRight).cellTitle(onTitle).colHeaderClick(handleColClick).itemCellClick(handleItemCellClick);
  
  ourModel = new textmodel.TextHash(loadedText, caseSensitive, fldNames, fldDelim, distinguishingFldsArray); //see http://www.sfs.uni-tuebingen.de/~cculy/software/DoubleTreeJS/index.html
  
    document.getElementById("words").value = "";

  //fill in datalist with the words we found
  uniqItems = ourModel.getUniqItemsWithCounts();
  document.getElementById("wordlist").innerHTML = '<select id="wordlistSelect">' + uniqItems.reduce(function(prev,curr){
      var val =  curr.split("\t")[0].replace(/<unknown>/g, '&lt;unknown&gt;').replace(/"/g,'&quot;');
      return prev + '<option value="' + val + '">' + val + '</option>\n';
  },"")  + '</select>'; //we need <select> for safari, since it doesn't support <datalist> yet;
  
  
  document.getElementById("searchLabel").style.visibility = "visible";
  document.getElementById("words").style.visibility = "visible";
  document.getElementById("wordlistSelect").style.visibility = "visible";
  document.getElementById("makeKWIC").style.visibility = "visible";
  document.getElementById("wordlistSelect").selectedIndex = -1;
  
}


function makeKWICandUpdate() {
  hideFullText();
  
  var searchTerm = document.getElementById("words").value;
  if (searchTerm === "") {
    //in Safari
    var opt = document.getElementById("wordlistSelect").selectedOptions;
    if (opt.length > 0) {
      searchTerm = opt[0].value;
    }
  }
  
  makeKWIC(searchTerm);
  pushHistory(searchTerm);
  if (kwic.hitsCount() === 0) {
    clearKWIC();
    return;
  }
  
  document.getElementById("kwicDiv").style.visibility ="visible";
  document.getElementById("kwicControls").style.visibility ="visible";
  
  
  if (kwic.actualFields().has('POS')) {
    document.getElementById("sortByPOS").style.visibility ="visible";
    document.getElementById("sortByPOSLabel").style.visibility ="visible";
  } else {
    document.getElementById("sortByPOS").style.visibility ="hidden";
    document.getElementById("sortByPOSLabel").style.visibility ="hidden";
  }
  
  if (document.getElementById("sortBySrc")) {
    if (kwic.actualFields().has('source')) {
      document.getElementById("sortBySrc").style.visibility ="visible";
      document.getElementById("sortBySrcLabel").style.visibility ="visible";
    } else {
      document.getElementById("sortBySrc").style.visibility ="hidden";
      document.getElementById("sortBySrcLabel").style.visibility ="hidden";
    }
  }
  
  if (document.getElementById("tok")) {
    document.getElementById("tok").disabled = searchTerm.indexOf('<unknown>') == -1;
  }
  
  updateShowing();
  setupReverse();
  
}

function reportPOS(item) {
  if (!item) {
    return;
  }
  if (uniqItems[0].indexOf(fldDelim) == -1) {
    return "No POS information for this corpus.";
  }
  if (caseSensitive) {
    var searchItem = item + fldDelim;
  } else {
    searchItem = item.toLocaleLowerCase() + fldDelim;
  }
  
  var justOurs = uniqItems.filter(function(elt){
      return elt.indexOf(searchItem) === 0;
      })
      .map(function(elt) {
          //e.g.   "one/CD 387"
          var pieces = elt.split("\t");
          return {"POS": pieces[0].split(fldDelim)[1], "count":+pieces[1]};
      })
      .sort(function(a,b) {
        return b.count - a.count;  
      })
      .reduce(function(prev, curr){
        return prev + curr.POS + "\t" + curr.count + "\n";
      }, "");
  
  return item + "\n" + justOurs;
  
}

/////////// more pure UI related functions

function handleFiles(files, fileHandler) {
  
  var file = files[0];
  var name = file.name;
  srcName = name;
  
  if (file.type != "text/plain") {
    alert(name + " is not a plain text file. Please select a plain text file instead.");
    document.getElementById("file").value = "";
    return null;
  }
  document.getElementById("loading").innerHTML = "Loading ...";
  var reader = new FileReader();
  reader.readAsText(file);
  
  reader.onerror = function(evt) {
    alert("There was a problem reading " + name);
    document.getElementById("loading").innerHTML = "";
  };
  
  reader.onabort = function(evt) {
    alert('File read cancelled');
    document.getElementById("loading").innerHTML = "";
  };
  
  reader.onloadstart = function(e) {
    newText = null;
  };
  reader.onload = function(e) {
    if (typeof(kwic) !== 'undefined') {
      try {
        hideFullText();
        clearKWIC();
      } catch (e) {}
    }
    clearHistory();
    fileHandler( reader.result );
    document.getElementById("loading").innerHTML = "";
  }
  return null;      
}

function handlePress(e) {
  var elt = e.originalTarget || e.target;
  if (e.keyCode == 13 && elt.id == "words") {
    makeKWICandUpdate();
  }
}

function clearKWIC() {
  document.getElementById("kwicDiv").style.visibility = 'hidden';
  var oldNum = kwic.numToShow();
  kwic.numToShow(0);
  kwic.showFirstPage();
  kwic.numToShow(oldNum);
  document.getElementById('kwicControls').style.visibility = 'hidden';
  document.getElementById("sortByPOS").style.visibility ="hidden";
  document.getElementById("sortByPOSLabel").style.visibility ="hidden";
  
}

function showNext() {
  kwic.showNextPage();
  updateShowing()
}

function showPrevious() {
  kwic.showPrevPage();
  updateShowing()
}

function showFirst() {
  kwic.showFirstPage();
  updateShowing();
}

function showLast() {
  kwic.showLastPage();
  updateShowing();
}


function updateShowing() {
  var numHits = kwic.hitsCount();
  var what;
  if (numHits === 0) {
    what = "No results;"
  } else {
    var showing = kwic.numToShow();
    var startElt = kwic.whichPage() * showing +1;
    var endElt = Math.min(startElt + showing -1, numHits );
    what = "Showing results " + startElt + " to " + endElt + " of " + numHits + " results";
  }
  document.getElementById("numHits").innerHTML = what;
}

function setupReverse() {
  //set up sortReversed
  for(var i=0;i<2*kwicContext +1;i++) {
    sortReversed[i] = true; //we are unsorted initially, so when we sort the first time it should NOT be reversed
  }

}


function hideFullText() {
  document.getElementById("fulltext").style.display = "none";
}

function setSearchTerm(item) {
    var sel = document.getElementById("wordlistSelect");
    var opts = sel.options;
    var vals = [].slice.call(opts).map(function(d) {return d.value});
    var which = vals.indexOf(item);
    opts[which].selected = true;
    if (haveWordsElt) {
      document.getElementById("words").value = item;
    }
    
}
