/* (This is the new BSD license.)
* Copyright (c) 2013-2016, Chris Culy
* All rights reserved.
*
* Redistribution and use in source and binary forms, with or without
* modification, are permitted provided that the following conditions are met:
*     * Redistributions of source code must retain the above copyright
*       notice, this list of conditions and the following disclaimer.
*     * Redistributions in binary form must reproduce the above copyright
*       notice, this list of conditions and the following disclaimer in the
*       documentation and/or other materials provided with the distribution.
*     * Neither the name of the Chris Culy nor the 
*		names of its contributors may be used to endorse or promote 
*		products from this software without specific prior written permission.
*
* THIS SOFTWARE IS PROVIDED BY Chris Culy
* ``AS IS'' AND ANY OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, 
* THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE 
* ARE DISCLAIMED. IN NO EVENT SHALL Chris Culy
* BE LIABLE FOR ANY, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR 
* CONSEQUENTIAL DAMAGES INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE 
* GOODS OR SERVICES; OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER 
* CAUSED AND ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR 
* TORT INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF 
* THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
*/

//overriding basic-common.js
var clusterer = new Clusterer();
var simMeasure = "cosine"; //"matching"; //"dice"; //"jaccard"; //"overlap_coefficient"; //"cosine";
var numClusters = 3;
var clusterColors;
resetColors();

function makeKWIC(searchTerm) {
  var toInclude = null;
  var isRegex = false;
  
  var arrays = ourModel.getItem(searchTerm, kwicContext, toInclude, isRegex);
  
  clusters = clusterer.data(arrays[0],arrays[1],arrays[2],arrays[3]).cluster(simMeasure, numClusters); //probably should extract first field for these arrays ?
    
  var tmu = textmodel.TextModelUtilities(fldNames, fldDelim);

  //convert the arrays to a single array of objects, so we can do sorting by position
  var realArray = [];
  var numHits = arrays[1].length;
  for (var i=0;i<numHits;i++) {
    var id = arrays[3][i];
    var hit = new tmu.TextInfo(arrays[1][i]);
    hit.cluster = clusters[id];
    
    realArray[i] = {'left': tmu.textArrayToTextInfo(arrays[0][i]),
                    'hit':  hit,
                    'right':tmu.textArrayToTextInfo(arrays[2][i]),
                    'index':id
                    };
    
  }
  
  kwic.data(realArray);
  kwic.showPage(1);
}

function reviseClusters() { 
  makeKWICandUpdate(); //for now, until we can be more clever; issue: clusterer expects strings, but we have converted input to objects in kwic
}

//overriding basic-common.js
function handleColClick(content, col) {
  
  if (content == "") {
    return; //do nothing if not clicking on hit or context
  }
  var settings = { };
  
  if (d3.event.altKey && content === "HIT") {
    settings['field'] = "cluster";
    settings['sortFun'] = function(a,b) {return a-b;};
  } else {
  
    var sbp = document.getElementById("sortByPOS");
    if (sbp && sbp.style.visibility == "visible" && sbp.checked) {
      settings['field'] = "POS";
    }
    
    var sbs = document.getElementById("sortBySrc");
    if (sbs && sbs.style.visibility == "visible" && sbs.checked) {
      settings['field'] = "source";
    }
  }
  
  //default is token (or baseFld)
  if (typeof(settings.field) == 'undefined') {
    if (kwic.actualFields().has("token")) {
      settings['field'] = "token";
    } else if (typeof(baseFld) != 'undefined') {
      settings['field'] = baseFld;
    } else {
      //??
    }
  }
  
  sortReversed[col] = ! sortReversed[col];
  settings['reverse'] = sortReversed[col];
  
  
  kwic.sort(col+1, settings);
  updateShowing();
  
}

//overriding basic-ui.js
function updateShowing() {
  var numHits = kwic.hitsCount();
  var what;
  if (numHits === 0) {
    what = "No results;"
  } else {
    var showing = kwic.numToShow();
    var startElt = kwic.whichPage() * showing +1;
    var endElt = Math.min(startElt + showing -1, numHits );
    what = "Showing results " + startElt + " to " + endElt + " of " + numHits + " results";
    
    ///cf filtered-ui.js
    what += ' <span id="distLabel" onclick="updateFilters();">Cluster distribution</span>: ';
    var agg = kwic.hitsByField("cluster");
    for (var clust in agg) {
      var howMany = agg[clust];
      var percent = Math.round(10000*howMany/numHits)/100;
      var wd = 12*howMany/numHits;
      what += '<span class="distBar" onclick="updateFilters(event,' + clust + ')" style="width:' + wd + 'em; background-color:' + clusterColors(clust) + ';" title="'+ howMany + ' (' + percent + '%)" >' + clust + '</span> ';
    }
  }
  document.getElementById("numHits").innerHTML = what;
  colorByCluster();
}


///new
function resetColors() {
  var numColors = numClusters == 2 ? 3 : numClusters;
  clusterColors = d3.scaleOrdinal()
    .range(colorbrewer.PiYG[numColors]);
}

function colorByCluster() {
  d3.selectAll("td.kwicTDHit")
    .style("background-color", function(d) {
      return clusterColors(d.cluster);
  });
}

//new
function updateFilters(e,clustNum) {
  var filters = kwic.filters();
  
  if (!e ) {
    filters.hit = null;
    
  } else if (e.altKey) {
    //remove clustNum
    var fun = kwic.exclude_filter_function('cluster', clustNum);
    var prevF = filters.hit;
    if (typeof(prevF) === 'function') {
      filters.hit = function(d) {
        return prevF(d) && fun(d);
      }
      //filters.hit = fun;
    } else {
      filters.hit = fun;
    }
    
    
  } else {
    //show only clustNum
    filters.hit = kwic.include_filter_function('cluster', clustNum);
  }
  
  kwic.filters(filters).showPage(1);
  updateShowing();
  
}