/* (This is the new BSD license for the original content. See below for included content)
* Copyright (c) 2014, Chris Culy
* All rights reserved.
*
* Redistribution and use in source and binary forms, with or without
* modification, are permitted provided that the following conditions are met:
*     * Redistributions of source code must retain the above copyright
*       notice, this list of conditions and the following disclaimer.
*     * Redistributions in binary form must reproduce the above copyright
*       notice, this list of conditions and the following disclaimer in the
*       documentation and/or other materials provided with the distribution.
*     * Neither the name of the Chris Culy nor the 
*		names of its contributors may be used to endorse or promote 
*		products from this software without specific prior written permission.
*
* THIS SOFTWARE IS PROVIDED BY Chris Culy
* ``AS IS'' AND ANY OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, 
* THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE 
* ARE DISCLAIMED. IN NO EVENT SHALL Chris Culy
* BE LIABLE FOR ANY, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR 
* CONSEQUENTIAL DAMAGES INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE 
* GOODS OR SERVICES; OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER 
* CAUSED AND ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR 
* TORT INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF 
* THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
*/

//does kMeans clustering
//for now, just on words, but could also try 2-3-4 ngrams
function Clusterer() {
    var fldSep = String.fromCharCode(31); //to distinguish left and right sides
    
    var contextType = "separate"; //default
    var contextLimit = 3; // default
    var useNgrams = false; //default
    
    var numItems;
    var vectors;
    var ids
    
    
    //setter/getter
    this.data = function(prefixes, hits, suffixes, _ids) {
        numItems = hits.length;
        ids = _ids;
        var arrays = {};
        var prefs, suffs;
        
        if (contextLimit) {
            prefs = prefixes.map(function(d) {
                return d.slice(-contextLimit);
            })
            suffs = suffixes.map(function(d) {
                return d.slice(0,contextLimit);
            })
        } else {
            prefs = prefixes;
            suffs = suffixes;
        }
        
        function addNgrams(eltArr) {
            //try 2,3,4 grams ?  2,3,4 was too many for "to" in poe
            var ns = [2,3];
            
            
            what = eltArr;
            var s = eltArr.join(" ");
            for(var i=0,n=ns.length;i<n;i++) {
                what = what.concat( ngrams(s, ns[i]) );
            }
            return what;
        }
        if (useNgrams) {
            prefs = prefs.map(addNgrams);
            suffs = suffs.map(addNgrams);
        }
        
        for(var i=0;i<numItems;i++) {
            var id = ids[i];
            if (! (id in arrays) ) {
                arrays[id] = [];
            }
            
            //enter the prefixes and suffixes *together* with hit into the vector
            if (contextType === "separate") {
                //mark prefixes and suffixes as distinct
                arrays[id] = arrays[id].concat(
                    prefs[i].map(function(d) {return "p" + fldSep + d;}),
                    hits[i],
                    suffs[i].map(function(d) {return "s" + fldSep + d;})
                );
                
            } else if (contextType === "leftOnly") {
                arrays[id] = arrays[id].concat(prefs[i], hits[i]);
            } else if (contextType === "rightOnly") {
                arrays[id] = arrays[id].concat(hits[i], suffs[i]);
            } else {
                arrays[id] = arrays[id].concat(prefs[i], hits[i], suffs[i]);
            }
        }
        prefs = null;
        suffs = null;
        
        vectors = {}
        for(var id in arrays) {
            vectors[id] = d3.set(arrays[id]).values();
        }
        return this;
    }
    
    //setter/getter
    this.contextType = function(value) {
        if (arguments.length === 0) {
            return contextType;
        }
        contextType = value;
        return this;
    }
    
    //setter/getter
    this.contextLimit = function(value)  {
        if (arguments.length === 0) {
            return contextLimit;
        }
        contextLimit = value;
        return this;
    }
    
    //setter/getter
    this.useNgrams = function(value) {
        if (arguments.length === 0) {
            return useNgrams;
        }
        useNgrams = value;
        return this;
    }
    
    this.cluster = function(simMeasure, numClusters) {
        simMeasure = simMeasure || "cosine"; //default is cosine
        numClusters = Math.max(numClusters,2); //default is 2 (and need minimum of 2)
        
        var simM = sim[simMeasure];
        
        //make nx1 array of similarities. Each element is object with id, pt, where pt is 1xn array of similarities
        var sims = [];
        
        //init
        for(var i=0;i<numItems;i++) {
            sims[i] = {id:ids[i], pt:[]};
        }
        for(var i=0;i<numItems;i++) {
            var id1 = ids[i];
            var v1 = vectors[id1];
            sims[i].pt[i] = 1;
            
            for (var j=0;j<i;j++) {
                var id2 = ids[j];
                var v2 = vectors[id2];
                
                var s = simM(v1,v2);
                sims[i].pt[j] = s;
                sims[j].pt[i] = s;
            }
        }
        
        //we can then feed that to kMeans
        var clusters = kmeans(sims, numClusters).filter(function(d) { return typeof(d) === 'object'; });
        
        //now make mapping from id to cluster number
        var what = {};
        for(var i=0, n=clusters.length;i<n;i++) {
            clusters[i].forEach(function(d){
               what[d.id] = i;
            });
        }
        return what;
    }
    
    //return all n-grams of s; no padding
    function ngrams(s,n) {
        var what = [];
        for(i=0,m=s.length-n+1;i<m;i++) {
            what.push( s.slice(i,i+n) );
        }
        return what;
    }


    //////////////// The kmeans clustering below has been *modified* by CuC from https://github.com/harthur/clusterfck, which has the MIT license given here ///////
    /*
    Copyright (c) 2011 Heather Arthur <fayearthur@gmail.com>
    
    Permission is hereby granted, free of charge, to any person obtaining
    a copy of this software and associated documentation files (the
    "Software"), to deal in the Software without restriction, including
    without limitation the rights to use, copy, modify, merge, publish,
    distribute, sublicense, and/or sell copies of the Software, and to
    permit persons to whom the Software is furnished to do so, subject to
    the following conditions:
    
    The above copyright notice and this permission notice shall be
    included in all copies or substantial portions of the Software.
    
    THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
    EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF
    MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND
    NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE
    LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION
    OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION
    WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
    */
    
    //CuC: distances were imported 
    var distances = {
      euclidean: function(v1, v2) {
          var total = 0;
          for (var i = 0; i < v1.length; i++) {
             total += Math.pow(v2[i] - v1[i], 2);      
          }
          return Math.sqrt(total);
       },
       manhattan: function(v1, v2) {
         var total = 0;
         for (var i = 0; i < v1.length ; i++) {
            total += Math.abs(v2[i] - v1[i]);      
         }
         return total;
       },
       max: function(v1, v2) {
         var max = 0;
         for (var i = 0; i < v1.length; i++) {
            max = Math.max(max , Math.abs(v2[i] - v1[i]));      
         }
         return max;
       }
    };
    
    
    function randomCentroids(points, k) {
        var centroids = points.slice(0); // copy
        centroids.sort(function() {
           return (Math.round(Math.random()) - 0.5);
        });
        return centroids.slice(0, k);
    }

    //CuC modifications:
    //  points are now objects with a "pt" key whose value is the vector. We do this so we can keep track of the different points after the clustering
    function closestCentroid(point, centroids, distance) {
       var min = Infinity,
           index = 0;
       for (var i = 0; i < centroids.length; i++) {
          var dist = distance(point.pt, centroids[i].pt);
          if (dist < min) {
             min = dist;
             index = i;
          }
       }
       return index;
    }
    
    //CuC modifications:
    //  points are now objects with a "pt" key whose value is the vector. We do this so we can keep track of the different points after the clustering
    function kmeans(points, k, distance, snapshotPeriod, snapshotCb) {
       k = k || Math.max(2, Math.ceil(Math.sqrt(points.length / 2)));
    
       distance = distance || "euclidean";
       if (typeof distance == "string") {
          distance = distances[distance];
       }
    
       var centroids = randomCentroids(points, k);
       var assignment = new Array(points.length);
       var clusters = new Array(k);
    
       var iterations = 0;
       var movement = true;
       while (movement) {
          // update point-to-centroid assignments
          for (var i = 0; i < points.length; i++) {
             assignment[i] = closestCentroid(points[i], centroids, distance);
          }
    
          // update location of each centroid
          movement = false;
          for (var j = 0; j < k; j++) {
             var assigned = [];
             for (var i = 0; i < assignment.length; i++) {
                if (assignment[i] == j) {
                   assigned.push(points[i]);
                }
             }
    
             if (!assigned.length) {
                continue;
             }
             var centroid = centroids[j];
             var newCentroid = new Array(centroid.length);
    
             for (var g = 0; g < centroid.length; g++) {
                var sum = 0;
                for (var i = 0; i < assigned.length; i++) {
                   sum += assigned[i][g];
                }
                newCentroid[g] = sum / assigned.length;
    
                if (newCentroid[g] != centroid[g]) {
                   movement = true;
                }
             }
             centroids[j] = newCentroid;
             clusters[j] = assigned;
          }
    
          if (snapshotCb && (iterations++ % snapshotPeriod == 0)) {
             snapshotCb(clusters);
          }
       }
       return clusters;
    }
}
