/* (This is the new BSD license.)
* Copyright (c) 2014, Chris Culy
* All rights reserved.
*
* Redistribution and use in source and binary forms, with or without
* modification, are permitted provided that the following conditions are met:
*     * Redistributions of source code must retain the above copyright
*       notice, this list of conditions and the following disclaimer.
*     * Redistributions in binary form must reproduce the above copyright
*       notice, this list of conditions and the following disclaimer in the
*       documentation and/or other materials provided with the distribution.
*     * Neither the name of the Chris Culy nor the 
*		names of its contributors may be used to endorse or promote 
*		products from this software without specific prior written permission.
*
* THIS SOFTWARE IS PROVIDED BY Chris Culy
* ``AS IS'' AND ANY OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, 
* THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE 
* ARE DISCLAIMED. IN NO EVENT SHALL Chris Culy
* BE LIABLE FOR ANY, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR 
* CONSEQUENTIAL DAMAGES INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE 
* GOODS OR SERVICES; OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER 
* CAUSED AND ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR 
* TORT INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF 
* THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
*/
"use strict";

rowStrOnRight = true;
var counts = {"P":0, "C":0, "M":0};

//just for saving a TextHash;
function tosave(loadedText) {
  ourModel = new textmodel.TextHash(loadedText, caseSensitive, fldNames, fldDelim, distinguishingFldsArray, baseFld );
  saveTextHash()
}

function init() {
  kwic = new kwicIS.KWIC("realKWIC");
  
  if (! fromTextInfo) {
    kwic = new kwicIS.KWIC("realKWIC").fieldNames(fldNames).fieldDelim(fldDelim);
  }
  kwic.contextLen(kwicContext).rowString(rowStr, rowStrOnRight).cellTitle(onTitle).colHeaderClick(handleColClick).itemCellClick(handleItemCellClick);
  
  ourModel = new textmodel.TextHash( "", caseSensitive, fldNames, fldDelim, distinguishingFldsArray, baseFld );
  if (rhx3) {
     
     for(var i=0,n=rhx3.items.length;i<n;i++) {
        var which = rhx3.items[i].charAt( rhx3.items[i].length -1 );
        counts[which]++;
     }
     ourModel.fromJSON(rhx3); //load the precalculated info
     rhx3 = null; //free up memory
  }
  
  document.getElementById("words").value = "";

  //fill in datalist with the words we found
  uniqItems = ourModel.getUniqItemsWithCounts();
  document.getElementById("wordlist").innerHTML = '<select id="wordlistSelect">' + uniqItems.reduce(function(prev,curr){
      var val =  curr.split("\t")[0].replace(/<unknown>/g, '&lt;unknown&gt;').replace(/"/g,'&quot;');
      return prev + '<option value="' + val + '">' + val + '</option>\n';
  },"")  + '</select>'; //we need <select> for safari, since it doesn't support <datalist> yet;
  
  
  document.getElementById("searchLabel").style.visibility = "visible";
  document.getElementById("words").style.visibility = "visible";
  document.getElementById("wordlistSelect").style.visibility = "visible";
  document.getElementById("makeKWIC").style.visibility = "visible";
  document.getElementById("wordlistSelect").selectedIndex = -1;
  
}
init();


//so we can save the TextHash as JSON during development for reloading in the production version
function saveTextHash() { 
    var toSave = JSON.stringify(ourModel); //safer to get value in debugger
    debugger;
}

function updateShowing() {
  var numHits = kwic.hitsCount();
  var what;
  if (numHits === 0) {
    what = "No results;"
  } else {
    var showing = kwic.numToShow();
    var startElt = kwic.whichPage() * showing +1;
    var endElt = Math.min(startElt + showing -1, numHits );
    what = "Showing results " + startElt + " to " + endElt + " of " + numHits + " results ";
    
    
    var agg = kwic.hitsByField("source");
    
    var ns = [], observed = [];
    for (var src in agg) {
      var howMany = agg[src];
      var percent = Math.round(10000*howMany/numHits)/100;
      //what += '<span style="padding-left:1em; padding-right:1em; background-color:' + colors(src) + ';">' + src + ': ' +howMany + ' (' + percent + '%) </span> ';
      var wd = 12*howMany/numHits;
      what += '<span style="display:inline-block; padding-left:1em; padding-right:1em; width:' + wd + 'em; background-color:' + colors(src) + ';" title="'+ howMany + ' (' + percent + '%)" >' + src + '</span> ';
      
      ns.push( counts[src] );
      observed.push(howMany);
    }
    
    if (observed.length > 1) {
      var p = simpleChiSquareProb(observed, ns);
      
      var sig = "Distibution ";
      if (p === null) {
        sig += " too small to test significance";
      } else if (p === "") {
         sig += " NOT significant at p=0.05";
      } else {
        sig += " significant at " + p
      }
      what += '<span id="prob">' + sig + ' (&chi;<sup>2</sup>)</span>'
    }
    
    
    
  }
  document.getElementById("numHits").innerHTML = what;
  
  colorBySrc();
}

var colors = d3.scaleOrdinal()
    .domain(["P","M","C"])
    .range(colorbrewer.Set2[3]);
    
  
function colorBySrc() {
  
  d3.selectAll("td.kwicTD")
    .style('background-color', function(d) {
      if (! d.source) {
        return; //e.g. for summary cells
      }
      return colors(d.source);
    });
}

function handlePress(e) {
  var elt = e.originalTarget || e.target;
  if (e.keyCode == 13 && ( elt.id == "words" || (e.shiftKey && (elt.id == 'author' || elt.id == 'tok' || elt.id == 'leftPOS' || elt.id == 'rtPOS'))) ) {
    //makeKWICandUpdate();
    updateFilters();
  }
}

function updateFilters() {
  document.getElementById("sortAndCtrls").style.visibility = "visible";
  makeKWICandUpdate();
  
  document.getElementById("kwicDiv").style.visibility ="visible";
  document.getElementById("kwicControls").style.visibility ="visible";
  
  
  if (kwic.hitsCount() === 0) {
    document.getElementById("sortAndCtrls").style.visibility = "hidden";
    document.getElementById("sortBySrc").style.visibility ="hidden";
    document.getElementById("sortBySrcLabel").style.visibility ="hidden";
  }

}
