/* (This is the new BSD license.)
* Copyright (c) 2014, Chris Culy
* All rights reserved.
*
* Redistribution and use in source and binary forms, with or without
* modification, are permitted provided that the following conditions are met:
*     * Redistributions of source code must retain the above copyright
*       notice, this list of conditions and the following disclaimer.
*     * Redistributions in binary form must reproduce the above copyright
*       notice, this list of conditions and the following disclaimer in the
*       documentation and/or other materials provided with the distribution.
*     * Neither the name of the Chris Culy nor the 
*		names of its contributors may be used to endorse or promote 
*		products from this software without specific prior written permission.
*
* THIS SOFTWARE IS PROVIDED BY Chris Culy
* ``AS IS'' AND ANY OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, 
* THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE 
* ARE DISCLAIMED. IN NO EVENT SHALL Chris Culy
* BE LIABLE FOR ANY, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR 
* CONSEQUENTIAL DAMAGES INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE 
* GOODS OR SERVICES; OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER 
* CAUSED AND ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR 
* TORT INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF 
* THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
*/
"use strict"
fldNames = ["token", "POS", "lemma", "source"];
distinguishingFldsArray = ["lemma", "POS"];
var baseFld = "lemma";
fldDelim = "/";


var kwicContext = 5;

function makeKWIC(searchTerm) {
  var toInclude = null;
  var isRegex = false;
  
  
  var author = document.getElementById("author").value.trim();
  var token = document.getElementById("tok").value.trim();
  var leftPOS = document.getElementById("leftPOS").value.trim();
  var rtPOS = document.getElementById("rtPOS").value.trim();
  
  var filters = {};
  var tokFilter;
  if (token.length > 0) {
    var re = new RegExp(token);
    tokFilter = function(d) {
      return re.test(d.token);
    }
  }
  
  var authorFilter;
  if (author.length > 0) {
    if (document.getElementById('limAuthor').selected) {
      authorFilter = kwic.include_filter_function('source', author)
    } else {
      authorFilter = kwic.exclude_filter_function('source', author)
    }
  }
  
  if (tokFilter) {
    if (authorFilter) {
      filters.hit = function(d) { return tokFilter(d) && authorFilter(d); }
    } else {
      filters.hit = tokFilter;
    }
  } else {
    if (authorFilter) {
      filters.hit = authorFilter;
    }
  }
  
  if (leftPOS.length > 0) {
    //filters.left = [kwic.include_filter_function('POS', leftPOS)];
    
    var exLPOS = leftPOS.split(",");
    var leftFilt = function(d) {
      return exLPOS.indexOf(d.POS) > -1;
    }
    filters.left = [leftFilt];
    
  }
  
  if (rtPOS.length > 0) {
    //filters.right = [kwic.include_filter_function('POS', rtPOS)];
    
    var exRPOS = rtPOS.split(",");
    var rtFilt = function(d) {
      return exRPOS.indexOf(d.POS) > -1;
    }
    filters.right = [rtFilt];
    
  }
  
   
  
  var arrays = ourModel.getItem(searchTerm, kwicContext, toInclude, isRegex);
  kwic.data(arrays).filters(filters);
  kwic.showPage(1);
  colorBySrc();
}


function onTitle(textInfo, i) {
  if (typeof textInfo === 'string') {
    return "";
  }
  var what = ""  
  for (var k in textInfo) {
    if (typeof textInfo[k] == 'function') {
      continue;
    }
    what += k + ': ' + textInfo[k] + '\n';
  }
  return what;
}


function handleItemCellClick(d,i) {
  if (d3.event.altKey) {  //but watch out for Firefox on Linux alternatively d3.event.shiftKey  
    if (typeof(d.index) !== 'undefined') {
      //i.e. we are on a hit
      showContext(d)
    }
  } else {
    if (typeof d.token !== 'undefined') {
      document.getElementById('words').value = d.lemma.toLowerCase() + fldDelim + d.POS;
      makeKWICandUpdate();
    }
  }
  
}

function showContext(d,i) {
  var what = '<hr><input type="button" value="x" onclick="hideFullText()"><h2>Result in context</h2><p>';
  
  var toGet = {};
  toGet[d.index] = true;
  
  var item = d.token;
  if (! caseSensitive) {
    item = item.toLowerCase();
  }
  var howMany = itemContextLen;
  if (d.POS) {
    item += fldDelim + d.POS;
    howMany = itemContextLen /2;
  }
 
  var results = ourModel.getItem(item,howMany,toGet, false);
  
  
  if (d.POS) {
    var context = results[0][0].join("<br>\n") + '<br>\n<span id="textHit">' + results[1][0] + '</span><br>\n' + results[2][0].join("<br>\n"); 
   
    
  } else {
    //plain text
    var context = results[0][0].join(" ") + ' <span id="textHit">' + results[1][0] + '</span> ' + results[2][0].join(" "); 
    context = context.replace(/ ([.:,;!?])/g, '$1'); //no spaces before certain puncutation
  }
  
  
  what +=  context + '</p>';
  document.getElementById("fulltext").innerHTML = what;
  document.getElementById("fulltext").style.display = "block";
  
  window.location = '#textHit';
  
  
}