/* (This is the new BSD license.)
* Copyright (c) 2014, Chris Culy
* All rights reserved.
*
* Redistribution and use in source and binary forms, with or without
* modification, are permitted provided that the following conditions are met:
*     * Redistributions of source code must retain the above copyright
*       notice, this list of conditions and the following disclaimer.
*     * Redistributions in binary form must reproduce the above copyright
*       notice, this list of conditions and the following disclaimer in the
*       documentation and/or other materials provided with the distribution.
*     * Neither the name of the Chris Culy nor the 
*		names of its contributors may be used to endorse or promote 
*		products from this software without specific prior written permission.
*
* THIS SOFTWARE IS PROVIDED BY Chris Culy
* ``AS IS'' AND ANY OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, 
* THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE 
* ARE DISCLAIMED. IN NO EVENT SHALL Chris Culy
* BE LIABLE FOR ANY, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR 
* CONSEQUENTIAL DAMAGES INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE 
* GOODS OR SERVICES; OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER 
* CAUSED AND ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR 
* TORT INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF 
* THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
*/
function makeKWIC(searchTerm) {
  lastSearch = searchTerm;
  var toInclude = null;
  
  var arrays = ourModel.getItem(searchTerm, kwicContext);
  
  
  //convert the arrays to a single array of objects. We'll use our ReTextInfo object for hits, but the TextModelUtilities.TextInfo for the contexts
  var tmu = textmodel.TextModelUtilities(fldNames, fldDelim);
  var realArray = [];
  var numHits = arrays[1].length;
  for (var i=0;i<numHits;i++) {
    
    realArray[i] = {'left': tmu.textArrayToTextInfo(arrays[0][i]),
                    'hit':  new ReTextInfo(arrays[1][i], fldNames, fldDelim),
                    'right':tmu.textArrayToTextInfo(arrays[2][i]),
                    'index':arrays[3][i]
                    };
  }
  
  kwic.data(realArray);
  

  
  kwic.showPage(1);
}


function handleItemCellClick(d,i) {
  if (typeof(d.index) !== 'undefined') {
    var stuff = ourModel.getItemAndContext(lastSearch, d.index, strContextLen);
    
    
    var what = '<hr><input type="button" value="x" onclick="hideFullText()"><h2>Result in context</h2><p>';
    
    var linePosn = stuff.left.indexOf('\n'); //our record delimiter
    what += stuff.left.substr(linePosn+1);
    what += '<span id="textHit">' + stuff.item + '</span>'
    linePosn = stuff.right.lastIndexOf('\n'); //our record delimiter
    what += stuff.right.substring(0,linePosn);
    
    document.getElementById('fulltext').innerHTML = what.replace(/\n/g, '<br>') + '</p>';
    document.getElementById("fulltext").style.display = "block";
    
    window.location = '#textHit';
  } else if (typeof d.token !== 'undefined') {
    document.getElementById('words').value = '/^' + d.token + fldDelim + d.POS.replace(/\$/g,'\\$') + '$/m';
    makeKWICandUpdate();
  }
  
}

function onTitle(textInfo, i) {
  if (textInfo.POS) {
    return textInfo.POS
  }
  return ""
}


/////////////////

//Custom info object to handle instances where the hit is multiple items
//we'll use the extra info from the first element of the hit as the info for the whole hit
function ReTextInfo(text, fldNames, fldDelim) {
  var that = this;
  
  var isMulti = (text.indexOf('\n') > -1);
  if (isMulti) {
    var items = text.split('\n');
    
    var pieces = items[0].split(fldDelim);
    pieces.forEach(function(p,i) {
      that[ fldNames[i] ] = p;
    });
    
    that[ fldNames[0] ] = items.reduce(function(prev,curr) {
      return prev + curr.split(fldDelim)[0] + ' ';
    }, "");
    
    
  } else {
    var pieces = text.split(fldDelim);

    pieces.forEach(function(p,i) {
        that[ fldNames[i] ] = p;
    });
  }
  
  

  this.toString = function() {
    return this[ fldNames[0] ];
  }
}