/* (This is the new BSD license.)
* Copyright (c) 2013-2014, Chris Culy
* All rights reserved.
*
* Redistribution and use in source and binary forms, with or without
* modification, are permitted provided that the following conditions are met:
*     * Redistributions of source code must retain the above copyright
*       notice, this list of conditions and the following disclaimer.
*     * Redistributions in binary form must reproduce the above copyright
*       notice, this list of conditions and the following disclaimer in the
*       documentation and/or other materials provided with the distribution.
*     * Neither the name of the Chris Culy nor the 
*		names of its contributors may be used to endorse or promote 
*		products from this software without specific prior written permission.
*
* THIS SOFTWARE IS PROVIDED BY Chris Culy
* ``AS IS'' AND ANY OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, 
* THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE 
* ARE DISCLAIMED. IN NO EVENT SHALL Chris Culy
* BE LIABLE FOR ANY, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR 
* CONSEQUENTIAL DAMAGES INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE 
* GOODS OR SERVICES; OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER 
* CAUSED AND ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR 
* TORT INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF 
* THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
*/

var kwic;
var ourModel;

var sortReversed = []; //keep track of whether columns are sorted in ascending or descending order

var strContextLen = 500; //how much of the full context we'll show when clicking on a hit
var itemHistory = new SearchHistory();

//set up UI stuff
document.addEventListener("keypress",handlePress,false);

  

function setup(loadedText) {
  kwic = new kwicIS.KWIC("realKWIC").fieldNames(fldNames).fieldDelim(fldDelim);
  kwic.contextLen(kwicContext).cellTitle(onTitle).colHeaderClick(handleColClick).itemCellClick(handleItemCellClick);
  
  ourModel = new textmodel.RegExTextModel(loadedText); 
  
  document.getElementById("words").value = "";

  
  document.getElementById("searchLabel").style.visibility = "visible";
  document.getElementById("words").style.visibility = "visible";
  document.getElementById("makeKWIC").style.visibility = "visible";
 
  
}


function makeKWICandUpdate() {
  hideFullText();
  
  var re = document.getElementById("words").value;
  if (re === '') {
    //NB: Chrome will happily make a RexExp from the empty string, which is a non-advancing match anything, which then hangs
    alert("Please enter a search expression.")
    return;
  }
  if (re.indexOf('/') === 0) {
    var lastSlash = re.lastIndexOf('/');
    var re2 = re.substring(1, lastSlash);
    var flags = re.substr(lastSlash+1);
    searchTerm = new RegExp(re2, flags);
  } else {
    searchTerm = new RegExp(re);
  }
    
  
  makeKWIC(searchTerm);
  pushHistory(searchTerm);
  
  document.getElementById('kwicDiv').style.display = 'block';
  document.getElementById('kwicDiv').style.visibility = 'visible';
  document.getElementById("kwicControls").style.visibility ="visible";
  
  
  if (kwic.actualFields().has('POS')) {
    document.getElementById("sortByPOS").style.visibility ="visible";
    document.getElementById("sortByPOSLabel").style.visibility ="visible";
  } else {
    document.getElementById("sortByPOS").style.visibility ="hidden";
    document.getElementById("sortByPOSLabel").style.visibility ="hidden";
  }
  
  if (document.getElementById("sortBySrc")) {
    if (kwic.actualFields().has('source')) {
      document.getElementById("sortBySrc").style.visibility ="visible";
      document.getElementById("sortBySrcLabel").style.visibility ="visible";
    } else {
      document.getElementById("sortBySrc").style.visibility ="hidden";
      document.getElementById("sortBySrcLabel").style.visibility ="hidden";
    }
  }
  
  if (document.getElementById("tok")) {
    document.getElementById("tok").disabled = searchTerm.indexOf('<unknown>') == -1;
  }
  
  updateShowing();
  setupReverse();
  
}



/////////// more pure UI related functions

function handleFiles(files, fileHandler) {
  hideFullText();
  
  var file = files[0];
  var name = file.name;
  if (file.type != "text/plain") {
    alert(name + " is not a plain text file. Please select a plain text file instead.");
    document.getElementById("file").value = "";
    return null;
  }
  var reader = new FileReader();
  reader.readAsText(file);
  
  reader.onerror = function(evt) {
    alert("There was a problem reading " + name);
  };
  
  reader.onabort = function(evt) {
    alert('File read cancelled');
  };
  
  reader.onloadstart = function(e) {
    newText = null;
  };
  reader.onload = function(e) {
    if (typeof(kwic) !== 'undefined') {
      try {
        clearKWIC();
      } catch (e) {}
    }
    clearHistory();
    fileHandler( reader.result );
  }
  return null;      
}

function handlePress(e) {
  var elt = e.originalTarget || e.target;
  if (e.keyCode == 13 && elt.id == "words") {
    makeKWICandUpdate();
  }
}

function clearKWIC() {
  var oldNum = kwic.numToShow();
  kwic.numToShow(0);
  kwic.showFirstPage();
  kwic.numToShow(oldNum);
  document.getElementById('kwicControls').style.visibility = 'hidden';
  document.getElementById('kwicDiv').style.display = 'none';
  
}

function showNext() {
  kwic.showNextPage();
  updateShowing()
}

function showPrevious() {
  kwic.showPrevPage();
  updateShowing()
}

function showFirst() {
  kwic.showFirstPage();
  updateShowing();
}

function showLast() {
  kwic.showLastPage();
  updateShowing();
}


function updateShowing() {
  var numHits = kwic.hitsCount();
  var what;
  if (numHits === 0) {
    what = "No results;"
  } else {
    var showing = kwic.numToShow();
    var startElt = kwic.whichPage() * showing +1;
    var endElt = Math.min(startElt + showing -1, numHits );
    what = "Showing results " + startElt + " to " + endElt + " of " + numHits + " results";
  }
  document.getElementById("numHits").innerHTML = what;
}

function setupReverse() {
  //set up sortReversed
  for(var i=0;i<2*kwicContext +1;i++) {
    sortReversed[i] = true; //we are unsorted initially, so when we sort the first time it should NOT be reversed
  }

}

function hideFullText() {
  document.getElementById("fulltext").style.display = "none";
}

function setSearchTerm(item) {
  document.getElementById("words").value = item;
}

