/* (This is the new BSD license.)
* Copyright (c) 2014, Chris Culy
* All rights reserved.
*
* Redistribution and use in source and binary forms, with or without
* modification, are permitted provided that the following conditions are met:
*     * Redistributions of source code must retain the above copyright
*       notice, this list of conditions and the following disclaimer.
*     * Redistributions in binary form must reproduce the above copyright
*       notice, this list of conditions and the following disclaimer in the
*       documentation and/or other materials provided with the distribution.
*     * Neither the name of the Chris Culy nor the 
*		names of its contributors may be used to endorse or promote 
*		products from this software without specific prior written permission.
*
* THIS SOFTWARE IS PROVIDED BY Chris Culy
* ``AS IS'' AND ANY OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, 
* THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE 
* ARE DISCLAIMED. IN NO EVENT SHALL Chris Culy
* BE LIABLE FOR ANY, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR 
* CONSEQUENTIAL DAMAGES INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE 
* GOODS OR SERVICES; OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER 
* CAUSED AND ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR 
* TORT INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF 
* THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
*/

//settings for the WKIC
var kwicContext = 5;


//settings for the textModel; it's OK if we don't actually have a second field, unless a token is the same as the fldDelim
var caseSensitive = false;
var fldNames =["token"];
var fldDelim = "/";
var lastSearch;


function makeKWIC(searchTerm) {
  lastSearch = searchTerm;
  var toInclude = null;
  
  var arrays = ourModel.getItem(searchTerm, kwicContext);
  kwic.data(arrays);
  
  kwic.showPage(1);
}


/////////// UI related functions

//for reportPOS() see basic-ui.js

function onTitle(textInfo, i) {
  if (textInfo.POS) {
    return textInfo.POS
  }
  return ""
}

function rowStr(textInfoArray, i) {
  var re = /[.:,;!?]/;
  
  return (i+1) + ". " + textInfoArray.reduce(function(prev, curr) {
    if (curr) {
      if (re.test(curr.token) || curr.token == "'s") {
        return prev + curr.token; //don't put space before some punctuation
      }
      return prev + " " + curr.token;
    }
    return prev;
  },"");
}

function handleColClick(content, col) {
  
  if (content == "") {
    return; //do nothing if not clicking on hit or context
  }
  var settings = { };
  
  
  
  var sbp = document.getElementById("sortByPOS");
  if (sbp && sbp.style.visibility == "visible" && sbp.checked) {
    settings['field'] = "POS";
  }
  
  var sbs = document.getElementById("sortBySrc");
  if (sbs && sbs.style.visibility == "visible" && sbs.checked) {
    settings['field'] = "source";
  }
  
  //default is token (or baseFld)
  if (typeof settings.field == 'undefined') {
    if (kwic.actualFields().has("token")) {
      settings['field'] = "token";
    } else if (typeof baseFld != 'undefined') {
      settings['field'] = baseFld;
    } else {
      //??
    }
  }
  
  sortReversed[col] = ! sortReversed[col];
  settings['reverse'] = sortReversed[col];
  
  
  kwic.sort(col+1, settings);
  updateShowing();
  
}

function handleItemCellClick(d,i) {
  if (typeof(d.index) !== 'undefined') {  //i.e. we are on a hit
    var stuff = ourModel.getItemAndContext(lastSearch, d.index, strContextLen);
    
    
    var what = '<hr><input type="button" value="x" onclick="hideFullText()"><h2>Result in context</h2><p>';
    
    var spacePosn = stuff.left.indexOf(' ');
    what += stuff.left.substr(spacePosn+1);
    what += '<span id="textHit">' + stuff.item + '</span>'
    spacePosn = stuff.right.lastIndexOf(' ');
    what += stuff.right.substring(0,spacePosn);
    
    what = what.replace(/ ([.:,;!?])/g, '$1');
    
    document.getElementById('fulltext').innerHTML = what.replace(/\n/g, '<br>') + '</p>';
    document.getElementById("fulltext").style.display = "block";
    
    window.location = '#textHit';
  } else if (typeof d.token !== 'undefined') {
    document.getElementById('words').value = d.toString();
    makeKWICandUpdate();
  }
   
}


