/* (This is the new BSD license.)
* Copyright (c) 2014, Chris Culy
* All rights reserved.
*
* Redistribution and use in source and binary forms, with or without
* modification, are permitted provided that the following conditions are met:
*     * Redistributions of source code must retain the above copyright
*       notice, this list of conditions and the following disclaimer.
*     * Redistributions in binary form must reproduce the above copyright
*       notice, this list of conditions and the following disclaimer in the
*       documentation and/or other materials provided with the distribution.
*     * Neither the name of the Chris Culy nor the 
*		names of its contributors may be used to endorse or promote 
*		products from this software without specific prior written permission.
*
* THIS SOFTWARE IS PROVIDED BY Chris Culy
* ``AS IS'' AND ANY OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, 
* THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE 
* ARE DISCLAIMED. IN NO EVENT SHALL Chris Culy
* BE LIABLE FOR ANY, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR 
* CONSEQUENTIAL DAMAGES INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE 
* GOODS OR SERVICES; OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER 
* CAUSED AND ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR 
* TORT INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF 
* THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
*/

function SearchHistory() {
    var items = []; //string representations of real items
    var stored = {}; //keys are the elements in items, values are {item:origalItem, info:theInfo} 
    
    this.pushItem = function(item, info) {
        var itemStr = item.toString(); //redundant for strings, need for RegExps
        
        if (itemStr in stored) {
            var which = items.indexOf(itemStr);
            items.splice(which,1);
        }
        items.push(itemStr);
        stored[itemStr] = {'item':item, 'info':info};
    }
    
     //returns [item, info], but does not modifiy the history
    this.peekTopItem = function() {
        itemStr = items[ items.length-1 ];
        return [stored[itemStr]['item'], stored[itemStr]['info'] ];
    }
    
    //returns [item, info]
    this.popItem = function() {
        var itemStr = items.pop();
        var what = [stored[itemStr]['item'], stored[itemStr]['info'] ];
        delete stored[itemStr];
        return what;
    }
    
    //returns info or null
    this.removeItem = function(item) {
        var what = null;
        
        var itemStr = item.toString(); //redundant for strings, need for RegExps
        
        
        if (itemStr in stored) {
            var which = items.indexOf(which);
            items.splice(which,1);
            what = stored[itemStr]['info'];
            delete stored[itemStr];
        }
        return what;
    }
    
    this.removeAll = function() {
        items = [];
        stored = {};
    }
    
    //returns array of [item,info]; doesn't modify history
    this.getAll = function() {
        return items.map(function(d,i) {
            return [ stored[d]['item'], stored[d]['info'] ];
        });
    }
    
    //function isRegExp(e) {
    //    return (typeof(e) === 'object' && e instanceof RegExp);
    //}
}


