/* (This is the new BSD license.)
* Copyright (c) 2014, Chris Culy
* All rights reserved.
*
* Redistribution and use in source and binary forms, with or without
* modification, are permitted provided that the following conditions are met:
*     * Redistributions of source code must retain the above copyright
*       notice, this list of conditions and the following disclaimer.
*     * Redistributions in binary form must reproduce the above copyright
*       notice, this list of conditions and the following disclaimer in the
*       documentation and/or other materials provided with the distribution.
*     * Neither the name of the Chris Culy nor the 
*		names of its contributors may be used to endorse or promote 
*		products from this software without specific prior written permission.
*
* THIS SOFTWARE IS PROVIDED BY Chris Culy
* ``AS IS'' AND ANY OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, 
* THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE 
* ARE DISCLAIMED. IN NO EVENT SHALL Chris Culy
* BE LIABLE FOR ANY, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR 
* CONSEQUENTIAL DAMAGES INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE 
* GOODS OR SERVICES; OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER 
* CAUSED AND ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR 
* TORT INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF 
* THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
*/

//Given 2 arrays with unique values (i.e. effectively sets), calculate some similarity metrics (i.e. have binary vectors)
//cf Manning and Schütze p. 299


var sim = {};

(function(){
    
    var s1Size, s2Size;

    sim.matching = function(s1,s2) {
        return intersectionSize(s1,s2);
    }
    
    sim.dice = function(s1, s2) {
        return 2*intersectionSize(s1,s2)/(s1Size + s2Size);
    }
    
    sim.jaccard = function(s1,s2) {
        return intersectionSize(s1,s2) / unionSize(s1,s2);
    }
    
    sim.overlap_coefficient = function(s1,s2) {
        return intersectionSize(s1,s2) / Math.min(s1Size, s2Size);
    }
    
    sim.cosine = function(s1,s2) {
        return intersectionSize(s1,s2) / Math.sqrt(s1Size*s2Size);
    }
    
    ///////
    
    function intersectionSize(s1, s2) {
        
        s1Size = s1.length;
        s2Size = s2.length;
        
        var what = 0;
        var smaller, larger;
        if (s1Size < s2Size) {
            smaller = s1;
            larger = s2;
        } else {
            smaller = s2;
            larger = s1;
        }
        
        for(var i=0,n=smaller.length;i<n;i++) {
            if (larger.indexOf( smaller[i] ) > -1) {
               what++;
            }
        }
        return what;
    }
    
    function unionSize(s1, s2) {
        s1Size = s1.length;
        s2Size = s2.length;
        
        var u = {};
        
        for(var i=0,n=s1.length;i<n;i++) {
            u[s1[i]] = true;
        }
        for(var i=0,n=s2.length;i<n;i++) {
            u[s2[i]] = true;
        }
        return Object.keys(u).length;
    }
    
})();
