/* (This is the new BSD license.)
* Copyright (c) 2014-2016, Chris Culy
* All rights reserved.
*
* Redistribution and use in source and binary forms, with or without
* modification, are permitted provided that the following conditions are met:
*     * Redistributions of source code must retain the above copyright
*       notice, this list of conditions and the following disclaimer.
*     * Redistributions in binary form must reproduce the above copyright
*       notice, this list of conditions and the following disclaimer in the
*       documentation and/or other materials provided with the distribution.
*     * Neither the name of the Chris Culy nor the 
*		names of its contributors may be used to endorse or promote 
*		products from this software without specific prior written permission.
*
* THIS SOFTWARE IS PROVIDED BY Chris Culy
* ``AS IS'' AND ANY OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, 
* THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE 
* ARE DISCLAIMED. IN NO EVENT SHALL Chris Culy
* BE LIABLE FOR ANY, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR 
* CONSEQUENTIAL DAMAGES INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE 
* GOODS OR SERVICES; OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER 
* CAUSED AND ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR 
* TORT INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF 
* THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
*/"use strict";

var kwicIS = kwicIS || {};

(function() {
  kwicIS.KWIC = function(container) {
    function setupHeaderData() {
      headerData = [];
      for (var i = 0; i < leftContextLen; i++) {
        if (!prefixesOnRight) {
          headerData.push("-" + (leftContextLen - i));
        } else {
          headerData.push("+" + (leftContextLen - i));
        }
      }
      headerData.push("HIT");
      for (var i = 0; i < rtContextLen; i++) {
        if (!prefixesOnRight) {
          headerData.push("+" + (1 + i));
        } else {
          headerData.push("-" + (1 + i));
        }
      }
      if (haveRowString) {
        if (rowStringOnRight) {
          headerData.push("Summary");
        } else {
          headerData.unshift("Summary");
        }
      }
    }
    function isSummaryCol(i) {
      if (haveRowString) {
        if (rowStringOnRight) {
          return i == headers.size() - 1;
        } else if (i == 0) {
          return true;
        }
      }
      return false;
    }
    function calcLastPage() {
      numHits = data.length - excludedIDs.size();
      if (!data || numHits === 0) {
        lastPage = 0;
      } else {
        lastPage = Math.floor((numHits - 1) / numToShow);
      }
    }
    function filter() {
      excludedIDs = d3.set();
      var hitHasFilter = typeof filters.hit === "function";
      var haveFilter = hitHasFilter;
      var leftHasFilter = [];
      for (var i = 0, n = data[0].left.length; i < n; i++) {
        leftHasFilter[i] = typeof filters.left !== "undefined" && typeof filters.left[i] === "function";
        haveFilter = haveFilter || leftHasFilter[i];
      }
      var rightHasFilter = [];
      for (var i = 0, n = data[0].left.length; i < n; i++) {
        rightHasFilter[i] = typeof filters.right !== "undefined" && typeof filters.right[i] === "function";
        haveFilter = haveFilter || rightHasFilter[i];
      }
      if (!haveFilter) {
        calcLastPage();
        return;
      }
      for (var i = 0, n = data.length; i < n; i++) {
        var d = data[i];
        if (hitHasFilter && !filters.hit(d.hit)) {
          excludedIDs.add(d.id);
          continue;
        }
        var foundOne = false;
        for (var j = 0, m = d.left.length; j < m; j++) {
          if (leftHasFilter[j] && !filters.left[j](d.left[m - j - 1])) {
            excludedIDs.add(d.id);
            foundOne = true;
            break;
          }
        }
        if (foundOne) {
          continue;
        }
        for (var j = 0, m = d.right.length; j < m; j++) {
          if (rightHasFilter[j] && !filters.right[j](d.right[j])) {
            excludedIDs.add(d.id);
            break;
          }
        }
      }
      calcLastPage();
    }
    var where;
    var id;
    var data;
    var fromObjs = false;
    var fldNames = [ "token" ];
    var fldDelim = "/";
    var textInfoToString;
    var tmu = textmodel.TextModelUtilities(fldNames, fldDelim, textInfoToString);
    var filters = {};
    var excludedIDs = d3.set();
    var numHits;
    var leftContextLen = 5;
    var rtContextLen = 5;
    var numToShow = 20;
    var whichPage = 0;
    var lastPage;
    var sortCol = 0;
    var headerData;
    var headers;
    var cellTitle = function(textInfo, i) {};
    var colHeaderClick = function(headerContent, col) {};
    var itemCellClick = function(textInfo, i) {};
    var rowString = function(textInfoArray, i) {};
    var rowStringOnRight;
    var haveRowString = false;
    var prefixesOnRight = false;
    var kwic = this;
    if (typeof container == "string") {
      where = document.getElementById(container);
      id = container;
    } else {
      where = container;
      id = container.id;
      if (!id) {
        container.id = "kwikContainer33";
        id = container.id;
      }
    }
    var table = d3.select(where).append("table").attr("class", "kwicTable");
    table.append("thead").append("tr");
    table.append("tbody");
    setupHeaderData();
    this.data = function(input) {
      if (!arguments.length) {
        return data;
      }
      if (typeof input[0] === "object" && input[0] instanceof Array) {
        data = [];
        numHits = input[1].length;
        for (var i = 0; i < numHits; i++) {
          if (!prefixesOnRight) {
            data[i] = {
              left: tmu.textArrayToTextInfo(input[0][i]),
              hit: new tmu.TextInfo(input[1][i]),
              right: tmu.textArrayToTextInfo(input[2][i]),
              index: input[3][i],
              id: i
            };
          } else {
            var leftTI = tmu.textArrayToTextInfo(input[2][i]);
            leftTI.reverse();
            var rtTI = tmu.textArrayToTextInfo(input[0][i]);
            rtTI.reverse();
            data[i] = {
              left: leftTI,
              hit: new tmu.TextInfo(input[1][i]),
              right: rtTI,
              index: input[3][i],
              id: i
            };
          }
        }
      } else {
        numHits = input.length;
        data = input.map(function(d, i) {
          d.id = i;
          if (prefixesOnRight) {
            d.left.reverse();
            d.right.reverse();
            var tmp = d.left;
            d.right = d.left;
            d.left = tmp;
          }
          return d;
        });
      }
      filters = {};
      excludedIDs = d3.set();
      calcLastPage();
      return kwic;
    };
    this.filteredIDs = function() {
      var what = [];
      for (var i = 0, n = data.length; i < n; i++) {
        var id = d.id;
        if (!excludedIDs.has(d.id)) {
          what.push(id);
        }
      }
      return what;
    };
    this.filters = function(value) {
      if (!arguments.length) {
        return filters;
      }
      filters = value;
      filter();
      return kwic;
    };
    this.fieldNames = function(value) {
      if (!arguments.length) {
        return fldNames;
      }
      fldNames = value;
      tmu = textmodel.TextModelUtilities(fldNames, fldDelim, textInfoToString);
      return kwic;
    };
    this.fieldDelim = function(value) {
      if (!arguments.length) {
        return fldDelim;
      }
      fldDelim = value;
      tmu = textmodel.TextModelUtilities(fldNames, fldDelim, textInfoToString);
      return kwic;
    };
    this.textInfoToString = function(value) {
      if (!arguments.length) {
        return textInfoToString;
      }
      textInfoToString = value;
      tmu = textmodel.TextModelUtilities(fldNames, fldDelim, textInfoToString);
      return kwic;
    };
    this.numToShow = function(value) {
      if (!arguments.length) {
        return numToShow;
      }
      numToShow = value;
      calcLastPage();
      return kwic;
    };
    this.whichPage = function(value) {
      if (!arguments.length) {
        return whichPage;
      }
      whichPage = value;
      return kwic;
    };
    this.leftContextLen = function(value) {
      if (!arguments.length) {
        return leftContextLen;
      }
      leftContextLen = value;
      setupHeaderData();
      return kwic;
    };
    this.rtContextLen = function(value) {
      if (!arguments.length) {
        return rtContextLen;
      }
      rtContextLen = value;
      setupHeaderData();
      return kwic;
    };
    this.contextLen = function(value) {
      if (!arguments.length) {
        return rtContextLen;
      }
      leftContextLen = value;
      rtContextLen = value;
      setupHeaderData();
      return kwic;
    };
    this.rtToLeft = function(value) {
      if (!arguments.length) {
        return prefixesOnRight;
      }
      prefixesOnRight = value;
      return kwic;
    };
    this.hitsCount = function() {
      return numHits;
    };
    this.hitsByField = function(field) {
      var what = {};
      data.filter(function(d) {
        return !excludedIDs.has(d.id);
      }).forEach(function(d) {
        var which = d.hit[field];
        if (typeof what[which] === "undefined") {
          what[which] = 0;
        }
        what[which]++;
      });
      return what;
    };
    this.actualFields = function() {
      if (typeof data != "undefined" && data.length > 0) {
        return d3.set(Object.keys(data[0].hit));
      }
      return d3.set();
    };
    this.cellTitle = function(theFun) {
      cellTitle = theFun;
      return kwic;
    };
    this.colHeaderClick = function(theFun) {
      colHeaderClick = theFun;
      return kwic;
    };
    this.itemCellClick = function(theFun) {
      itemCellClick = theFun;
      return kwic;
    };
    this.rowString = function(theFun, onRight) {
      rowString = theFun;
      if (typeof onRight == "boolean") {
        rowStringOnRight = onRight;
      } else {
        rowStringOnRight = true;
      }
      haveRowString = true;
      setupHeaderData();
      return kwic;
    };
    this.sort = function(column, settings) {
      settings = settings || {};
      var field;
      var reverse = false;
      var sortFun;
      if (settings.hasOwnProperty("field")) {
        field = settings["field"];
      }
      if (settings.hasOwnProperty("reverse")) {
        reverse = settings["reverse"];
      }
      if (settings.hasOwnProperty("sortFun")) {
        sortFun = settings["sortFun"];
      }
      if (typeof sortFun === "undefined") {
        sortFun = function(a, b) {
          var lowerA = a.toLocaleLowerCase();
          var lowerB = b.toLocaleLowerCase();
          return lowerA < lowerB ? -1 : lowerA > lowerB ? 1 : 0;
        };
      }
      var hitCol = leftContextLen + 1;
      if (haveRowString && !rowStringOnRight) {
        column--;
      }
      data.sort(function(obj1, obj2) {
        var elt1, elt2;
        if (column == hitCol) {
          elt1 = obj1["hit"];
          elt2 = obj2["hit"];
        } else if (column < hitCol) {
          var which = column - 1;
          elt1 = obj1["left"][which];
          elt2 = obj2["left"][which];
        } else {
          var which = column - (leftContextLen + 2);
          elt1 = obj1["right"][which];
          elt2 = obj2["right"][which];
        }
        var comp1, comp2;
        if (typeof elt1 === "undefined") {
          comp1 = "";
        } else {
          if (typeof field !== "undefined") {
            if (typeof elt1[field] === "undefined") {
              comp1 = "";
            } else {
              comp1 = elt1[field];
            }
          } else {
            comp1 = elt1.toString();
          }
        }
        if (typeof elt2 === "undefined") {
          comp2 = "";
        } else {
          if (typeof field !== "undefined") {
            if (typeof elt2[field] === "undefined") {
              comp2 = "";
            } else {
              comp2 = elt2[field];
            }
          } else {
            comp2 = elt2.toString();
          }
        }
        return sortFun(comp1, comp2);
      });
      if (reverse) {
        data.reverse();
      }
      kwic.showPage(1);
    };
    this.showPage = function(pageNum) {
      kwic.whichPage(pageNum - 1).show();
    };
    this.showNextPage = function() {
      if (whichPage == lastPage) {
        return;
      }
      whichPage += 1;
      kwic.show();
    };
    this.showPrevPage = function() {
      if (whichPage == 0) {
        return;
      }
      kwic.whichPage(kwic.whichPage() - 1).show();
    };
    this.showFirstPage = function() {
      kwic.whichPage(0).show();
    };
    this.showLastPage = function() {
      kwic.whichPage(lastPage).show();
    };
    this.show = function() {
      var startElt = numToShow * whichPage;
      var thisData = data.filter(function(d) {
        return !excludedIDs.has(d.id);
      }).slice(startElt, startElt + numToShow).map(function(d, i) {
        d.left = d.left.slice(-leftContextLen);
        while (d.left.length < leftContextLen) {
          d.left.unshift("");
        }
        d.right = d.right.slice(0, rtContextLen);
        while (d.right.length < rtContextLen) {
          d.right.push("");
        }
        var what = d.left.concat([ d.hit ], d.right);
        var rowStr;
        if (!prefixesOnRight) {
          rowStr = [ rowString(what, startElt + i) ];
        } else {
          var tmpWhat = what.slice();
          tmpWhat.reverse();
          rowStr = [ rowString(tmpWhat, startElt + i) ];
        }
        if (haveRowString) {
          if (rowStringOnRight) {
            what = what.concat(rowStr);
          } else {
            what = rowStr.concat(what);
          }
        }
        return {
          cells: what,
          index: d.index,
          id: d.id
        };
      });
      headers = d3.select(where).select(" table > thead > tr").selectAll("th").data(headerData);
      headers.enter().append("th").text(String).classed("kwicTH", true).classed("kwicTHHit", function(d, i) {
        return d == "HIT";
      }).classed("kwicRowSummary", function(d, i) {
        return isSummaryCol(i);
      }).on("click", colHeaderClick);
      var tr = d3.select(where).select(" table > tbody").selectAll("tr").data(thisData);
      tr.enter().append("tr");
      tr.attr("id", function(d) {
        return d.id;
      }).attr("index", function(d) {
        return d.index;
      });
      tr.exit().remove();
      var td = tr.selectAll("td").data(function(d) {
        return d.cells;
      });
      td.enter().append("td").on("click", itemCellClick);
      td.html(function(d) {
        return d.toString();
      }).attr("class", "kwicTD").classed("kwicTDHit", function(d, i) {
        var isHit;
        if (!haveRowString || rowStringOnRight) {
          isHit = i == leftContextLen;
        } else if (haveRowString) {
          isHit = i == 1 + leftContextLen;
        }
        return isHit;
      }).classed("kwicRowSummary", function(d, i) {
        return isSummaryCol(i);
      }).attr("title", cellTitle);
      d3.select(where).selectAll(".kwicTDHit").each(function(d) {
        var p = d3.select(this.parentNode);
        d.id = +p.attr("id");
        d.index = p.attr("index");
        return d;
      });
      return;
    };
    this.include_filter_function = function(field, val) {
      return function(d) {
        return d[field] === val;
      };
    };
    this.exclude_filter_function = function(field, val) {
      return function(d) {
        return d[field] !== val;
      };
    };
  };
})();