/* (This is the new BSD license.)
* Copyright (c) 2014, Chris Culy
* All rights reserved.
*
* Redistribution and use in source and binary forms, with or without
* modification, are permitted provided that the following conditions are met:
*     * Redistributions of source code must retain the above copyright
*       notice, this list of conditions and the following disclaimer.
*     * Redistributions in binary form must reproduce the above copyright
*       notice, this list of conditions and the following disclaimer in the
*       documentation and/or other materials provided with the distribution.
*     * Neither the name of the Chris Culy nor the 
*		names of its contributors may be used to endorse or promote 
*		products from this software without specific prior written permission.
*
* THIS SOFTWARE IS PROVIDED BY Chris Culy
* ``AS IS'' AND ANY OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, 
* THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE 
* ARE DISCLAIMED. IN NO EVENT SHALL Chris Culy
* BE LIABLE FOR ANY, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR 
* CONSEQUENTIAL DAMAGES INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE 
* GOODS OR SERVICES; OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER 
* CAUSED AND ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR 
* TORT INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF 
* THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
*/

"use strict";
var textmodel = textmodel || {};

(function(){

/**
  * @class textmodel.TextModelUtilities
  *  This is the class for the TextModelUtilities
  * <p>
  * The constructor
  * @param _fldNames is an array of strings of field names
  * @param _fldDelim is a string containing the field delimiter
  * @param tiToString is a function that will become the toString method of {@link textmodel.TextModelUtilities.TextInfo}
  */   
textmodel.TextModelUtilities = function(_fldNames, _fldDelim, tiToString) {
    
    var what = this;
    var fldNames = _fldNames;
    var fldDelim = _fldDelim;
    
    /**
     * Convenience function that recursively creates TextInfo objects from (arrys of) array elments
     * @param arrayOfText is an array of delimited text, or an array of arrays of delimited text
     * @returns corresponding array(s) of TextInfo objects
     */
    this.textArrayToTextInfo = function(arrayOfText) {
        var elt = arrayOfText[0]; //assuming this exists
        if (typeof elt === "object" && elt instanceof Array) { //cf. http://javascript.crockford.com/remedial.html
            return arrayOfText.map(function(e) {
                return what.textArrayToTextInfo(e);    
            });
        }
        
        return arrayOfText.map(function(t) {
            return new what.TextInfo(t);    
        });
    }
    
    /**
     * Constructor for an object derived from delimited text e.g. dogs/dog/NN -> {token:"dogs", lemma:"dog", pos:"NN"}
     * @param text is the delmited text to convert into an object. The field names and delimiter come from {@link trextModel.textModelUtilities}
     */
    this.TextInfo = function(text) {
        var that = this;
        
        var pieces = text.split(fldDelim);
    
        pieces.forEach(function(p,i) {
            that[ fldNames[i] ] = p;
        });
       
        
    }
    
    if (typeof(tiToString) === 'function') {
        this.TextInfo.prototype.toString = tiToString;
    } else {
        this.TextInfo.prototype.toString = function() {
            return this[ fldNames[0] ];
        }
    }
    
    return this;
}
    
})();